/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
import  com.examples.nodatabinding.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.IOException;

/**
 * Client to demonstrate using the JAX-RPC SI's -f:nodatabinding switch
 * This example client uses the nodatabinding switch and thus the generated method signatures have a
 * SOAPElement which would not exist if the client was generated using the default options
 */
public class NoBindingClient {

    public static void main(String[] args) throws Exception {
        String url = "http://localhost:9090/nodatabinding/jaxrpc";
        if (args.length == 1) {
            url = args[0];
        }
        POService_Impl serviceproxy = new POService_Impl();
        IPurchaseOrder_Stub stub = (IPurchaseOrder_Stub) (serviceproxy.getIPurchaseOrderPort());
        stub._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, url);
        SOAPElement payload = createSOAPMessage("purchaseorder.xml");
        SOAPElement response = stub.acceptPO(payload);
        printNodeToConsole(response);


    }


    /**
     * Method to create a SOAPElement from a file
     *
     * @return SOAPElement
     * @throws Exception
     */
    public static SOAPElement createSOAPMessage(String filename) throws Exception {

        Document doc = readFileCreateDocument(filename);
        if (doc == null)
            throw new IOException("Problem reading cand creating DOM from file,readFileCreateDocument() returned null ");

        SOAPElement parent = SOAPFactory.newInstance().createElement("dummy");

        TransformerFactory factory = TransformerFactory.newInstance();
        Transformer transformer = factory.newTransformer();
        // also print to console
        transformer.transform(new DOMSource(doc), new DOMResult(parent));
        return (SOAPElement) parent.getChildElements().next();
    }


    /**
     * Read a file and create a DOM out of it
     *
     * @param filename the file where the XML is stored
     * @return
     */
    private static Document readFileCreateDocument(String filename) {
        Document xml;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
// Use the factory to create a DOM parser
            DocumentBuilder parser = factory.newDocumentBuilder();
// Create a new handler to handle content
//            parser.setErrorHandler(new MyErrorHandler());
            xml = parser.parse(new File(filename));
            return xml;
        } catch (Exception e) {
            System.out.println("Couldnt read file " + filename + " " + e);
        }
        return null;
    }

    public static void printNodeToConsole(Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }


}
